@@ -40,7 +40,7 @@ void AppleiSubEngine::closeiSub (IOService * forClient) {
 				result = audioEngine->removeDefaultAudioControl (iSubVolumeControl);
 				iSubVolumeControl->release ();
 				iSubVolumeControl = NULL;
-				#if DEBUGLOG
+				#if DEBUGLOGGING
 				if (kIOReturnSuccess != result) debugIOLog ("Error 0x%x removing left iSub control", result);
 				#endif
 			}
@@ -50,7 +50,7 @@ void AppleiSubEngine::closeiSub (IOService * forClient) {
 				//muteControl->release ();
 				muteControl->release ();
 				muteControl = NULL;
-				#if DEBUGLOG
+				#if DEBUGLOGGING
 				if (kIOReturnSuccess != result) debugIOLog ("Error 0x%x removing mute iSub control", result);
 				#endif
 			}
@@ -58,7 +58,7 @@ void AppleiSubEngine::closeiSub (IOService * forClient) {
 			release ();		// It's safe to release us now, there are no more controls that might be able to call into us.
 			// Don't try to use our audio engine host anymore
 			audioEngine = NULL;
-#if DEBUGLOG
+#if DEBUGLOGGING
 		} else {
 			if (NULL == muteControl) {debugIOLog ("NULL == muteControl");}
 			if (NULL == iSubVolumeControl) {debugIOLog ("NULL == iSubVolumeControl");}
@@ -654,7 +654,7 @@ void AppleiSubEngine::WriteHandler (void * object, void * buffer, IOReturn resul
 	SUB_ABSOLUTETIME (&t_now, &t_old);		// t_now -= t_old;
 	absolutetime_to_nanoseconds (t_now, &nanos_del);
 
-#if DEBUGLOG
+#if DEBUGLOGGING
 	nanos_del = nanos_del / (1000);
 	if (nanos_del > 11000) {
 		debugIOLog ("iSubEngine::WriteHandler - delta = %ld microseconds", (UInt32)(nanos_del));
