@@ -59,20 +59,25 @@ Exit:
     return configObject;
 }
 
-#if DEBUGLOG
+#if DEBUGLOGGING
 void USBAudioConfigObject::DumpConfigMemoryToIOLog (void) {
-#if 0
+#if 1
 	UInt8 *	descriptorPtr;
 	UInt8	descriptorIndex;
 	UInt8	length;
+	char	descriptor[256];
+	char	num[4];
 
 	descriptorPtr = (UInt8*)theConfigurationDescriptorPtr;
 	while (*descriptorPtr != 0) {
 		length = *descriptorPtr;
+		descriptor[0] = 0;
 		for (descriptorIndex = 0; descriptorIndex < length; descriptorIndex++) {
-			debugIOLog ("%2x ", *descriptorPtr++);
+			sprintf (num, "%2x ", *descriptorPtr++);
+			strcat (descriptor, num);
 		}
-		debugIOLog ("");
+		descriptor[length * 3] = 0;
+		debugIOLog ("%s", descriptor);
 	}
 #endif
 }
@@ -93,7 +98,7 @@ bool USBAudioConfigObject::init (const IOUSBConfigurationDescriptor * newConfigu
 
     memcpy (theConfigurationDescriptorPtr, newConfigurationDescriptor, USBToHostWord (newConfigurationDescriptor->wTotalLength));
     ((UInt8 *)theConfigurationDescriptorPtr)[USBToHostWord (newConfigurationDescriptor->wTotalLength)] = 0;
-#if DEBUGLOG
+#if DEBUGLOGGING
 	DumpConfigMemoryToIOLog ();
 #endif
 
@@ -255,8 +260,10 @@ UInt8 USBAudioConfigObject::GetFeatureUnitIDConnectedToOutputTerminal (UInt8 int
     USBAudioControlObject *				thisControl;
     UInt8								featureUnitID;
 
+	debugIOLog ("GetFeatureUnitIDConnectedToOutputTerminal (%d, %d, %d)", interfaceNum, altInterfaceNum, outputTerminalID);
     featureUnitID = 0;
     thisControl = GetControlObject (interfaceNum, altInterfaceNum);
+	debugIOLog ("GetControlObject returns %p", thisControl);
     if (thisControl)
 		featureUnitID = thisControl->GetFeatureUnitIDConnectedToOutputTerminal (outputTerminalID);
 
@@ -635,7 +642,6 @@ UInt8 USBAudioConfigObject::GetNumAltStreamInterfaces (UInt8 interfaceNum) {
 
     numAltInterfaces = 0;
 	if (NULL != theStreams) {
-		debugIOLog ("USBAudioConfigObject::GetNumAltInterfaces %d = theStreams->getCount ()", theStreams->getCount ());
 		for (indx = 0; indx < theStreams->getCount (); indx++) {
 			thisStream = OSDynamicCast (USBAudioStreamObject, theStreams->getObject (indx));
 			if (thisStream && thisStream->GetInterfaceNum () == interfaceNum) {
@@ -659,6 +665,18 @@ UInt8 USBAudioConfigObject::GetNumChannels (UInt8 interfaceNum, UInt8 altInterfa
     return numChannels;
 }
 
+UInt8 USBAudioConfigObject::GetNumControls (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 featureUnitID) {
+    USBAudioControlObject *				thisControl;
+    UInt8								numControls;
+
+	numControls = 0;
+    thisControl = GetControlObject (interfaceNum, altInterfaceNum);
+    if (thisControl)
+		numControls = thisControl->GetNumControls (featureUnitID);
+
+    return numControls;
+}
+
 UInt8 USBAudioConfigObject::GetNumSampleRates (UInt8 interfaceNum, UInt8 altInterfaceNum) {
     USBAudioStreamObject * 				thisStream;
     UInt8								numSampleRates;
@@ -695,16 +713,28 @@ UInt8 USBAudioConfigObject::GetNumOutputTerminals (UInt8 interfaceNum, UInt8 alt
 	return numOutputTerminals;
 }
 
-UInt8 USBAudioConfigObject::GetNumSelectorSources (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID) {
+UInt8 USBAudioConfigObject::GetNumSelectorUnits (UInt8 interfaceNum, UInt8 altInterfaceNum) {
     USBAudioControlObject * 			thisControl;
-	UInt8								numSelectorSources;
+	UInt8								numSelectorUnits;
 
-	numSelectorSources = 0;
+	numSelectorUnits = 0;
     thisControl = GetControlObject (interfaceNum, altInterfaceNum);
     if (thisControl)
-		numSelectorSources = thisControl->GetNumSelectorSources (unitID);
+		numSelectorUnits = thisControl->GetNumSelectorUnits ();
 
-	return numSelectorSources;
+	return numSelectorUnits;
+}
+
+UInt8 USBAudioConfigObject::GetNumSources (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID) {
+    USBAudioControlObject * 			thisControl;
+	UInt8								numSources;
+
+	numSources = 0;
+    thisControl = GetControlObject (interfaceNum, altInterfaceNum);
+    if (thisControl)
+		numSources = thisControl->GetNumSources (unitID);
+
+	return numSources;
 }
 
 UInt8 USBAudioConfigObject::GetNumStreamInterfaces (void) {
@@ -716,7 +746,6 @@ UInt8 USBAudioConfigObject::GetNumStreamInterfaces (void) {
     numInterfaces = 0;
     interfaceNum = 0;
 	if (NULL != theStreams) {
-		debugIOLog ("USBAudioConfigObject::GetNumInterfaces %d = theStreams->getCount ()", theStreams->getCount ());
 		for (indx = 0; indx < theStreams->getCount (); indx++) {
 			thisStream = OSDynamicCast (USBAudioStreamObject, theStreams->getObject (indx));
 			if (thisStream && thisStream->GetInterfaceNum () != interfaceNum) {
@@ -777,6 +806,54 @@ UInt8 * USBAudioConfigObject::GetSelectorSources (UInt8 interfaceNum, UInt8 altI
     return sources;
 }
 
+UInt8 USBAudioConfigObject::GetSubType (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID) {
+    USBAudioControlObject *				thisControl;
+    UInt8								subType;
+
+    subType = 0;
+    thisControl = GetControlObject (interfaceNum, altInterfaceNum);
+    if (thisControl)
+        subType = thisControl->GetSubType (unitID);
+
+    return subType;
+}
+
+UInt8 USBAudioConfigObject::GetSourceID (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID) {
+    USBAudioControlObject *				thisControl;
+    UInt8								sourceID;
+
+    sourceID = 0;
+    thisControl = GetControlObject (interfaceNum, altInterfaceNum);
+    if (thisControl)
+        sourceID = thisControl->GetSourceID (unitID);
+
+    return sourceID;
+}
+
+UInt8 USBAudioConfigObject::GetUnitType (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID) {
+    USBAudioControlObject *				thisControl;
+    UInt8								unitType;
+
+    unitType = 0;
+    thisControl = GetControlObject (interfaceNum, altInterfaceNum);
+    if (thisControl)
+        unitType = thisControl->GetSourceID (unitID);
+
+    return unitType;
+}
+
+UInt8 * USBAudioConfigObject::GetSourceIDs (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 unitID) {
+    USBAudioControlObject *				thisControl;
+    UInt8 *								sourceIDs;
+
+    sourceIDs = 0;
+    thisControl = GetControlObject (interfaceNum, altInterfaceNum);
+    if (thisControl)
+        sourceIDs = thisControl->GetSourceIDs (unitID);
+
+    return sourceIDs;
+}
+
 UInt8 USBAudioConfigObject::GetSubframeSize (UInt8 interfaceNum, UInt8 altInterfaceNum) {
     USBAudioStreamObject * 				thisStream;
     UInt8								subframeSize;
@@ -1172,9 +1249,31 @@ void USBAudioControlObject::free (void) {
 		theSelectorUnits = NULL;
 	}
 
+	if (NULL != theProcessingUnits) {
+		theProcessingUnits->release ();
+		theProcessingUnits = NULL;
+	}
+
+	if (NULL != theExtensionUnits) {
+		theExtensionUnits->release ();
+		theExtensionUnits = NULL;
+	}
+
     OSObject::free ();
 }
 
+UInt8 USBAudioControlObject::GetNumControls (UInt8 featureUnitID) {
+	USBFeatureUnitObject *				thisFeatureUnit;
+    UInt8								numControls;
+
+	numControls = 0;
+	thisFeatureUnit = GetFeatureUnitObject (featureUnitID);
+	if (thisFeatureUnit)
+        numControls = thisFeatureUnit->GetNumControls ();
+
+    return numControls;
+}
+
 // Channel #1 is left channel, #2 is right channel
 Boolean USBAudioControlObject::ChannelHasMuteControl (UInt8 featureUnitID, UInt8 channelNum) {
 	USBFeatureUnitObject *				thisFeatureUnit;
@@ -1202,16 +1301,20 @@ Boolean USBAudioControlObject::ChannelHasVolumeControl (UInt8 featureUnitID, UIn
 
 USBACDescriptorObject * USBAudioControlObject::GetACDescriptorObject (UInt8 unitID) {
     USBACDescriptorObject * 			thisACDescriptorObject;
-    UInt8								indx;
-    Boolean								done;
 
-    indx = 0;
-    done = FALSE;
 	thisACDescriptorObject = GetInputTerminalObject (unitID);
 	if (!thisACDescriptorObject)
 		thisACDescriptorObject = GetOutputTerminalObject (unitID);
 	if (!thisACDescriptorObject)
 		thisACDescriptorObject = GetMixerObject (unitID);
+	if (!thisACDescriptorObject)
+		thisACDescriptorObject = GetSelectorUnitObject (unitID);
+	if (!thisACDescriptorObject)
+		thisACDescriptorObject = GetFeatureUnitObject (unitID);
+	if (!thisACDescriptorObject)
+		thisACDescriptorObject = GetProcessingUnitObject (unitID);
+	if (!thisACDescriptorObject)
+		thisACDescriptorObject = GetExtensionUnitObject (unitID);
 
     return thisACDescriptorObject;
 }
@@ -1433,6 +1536,30 @@ USBInputTerminalObject * USBAudioControlObject::GetInputTerminalObject (UInt8 un
     return thisInputTerminal;
 }
 
+USBOutputTerminalObject * USBAudioControlObject::GetOutputTerminalObject (UInt8 unitID) {
+    USBOutputTerminalObject * 			thisOutputTerminal;
+    UInt8								indx;
+    Boolean								found;
+
+    indx = 0;
+    found = FALSE;
+	thisOutputTerminal = NULL;
+	if (NULL != theOutputTerminals) {
+		while (!found && indx < theOutputTerminals->getCount ()) {
+			thisOutputTerminal = OSDynamicCast (USBOutputTerminalObject, theOutputTerminals->getObject (indx));
+			if (thisOutputTerminal && unitID == thisOutputTerminal->GetUnitID ()) {
+				found = TRUE;
+			}
+			indx++;
+		}
+	}
+
+	if (found != TRUE)
+		thisOutputTerminal = NULL;
+
+    return thisOutputTerminal;
+}
+
 UInt16 USBAudioControlObject::GetInputTerminalType (UInt8 unitID) {
     USBInputTerminalObject *			thisInputTerminal;
     UInt16								terminalType;
@@ -1467,16 +1594,103 @@ UInt8 USBAudioControlObject::GetNumOutputTerminals (void) {
 	return count;
 }
 
-UInt8 USBAudioControlObject::GetNumSelectorSources (UInt8 unitID) {
-    USBSelectorUnitObject *				thisSelectorUnit;
-    UInt8								numSelectorSources;
+UInt8 USBAudioControlObject::GetNumSelectorUnits (void) {
+	UInt8								count;
 
-    numSelectorSources = 0;
-    thisSelectorUnit = GetSelectorUnitObject (unitID);
-    if (thisSelectorUnit)
-		numSelectorSources = thisSelectorUnit->GetNumInPins ();
+	count = 0;
+	if (NULL != theSelectorUnits) {
+		count = theSelectorUnits->getCount ();
+	}
 
-    return numSelectorSources;
+	return count;
+}
+
+UInt8 USBAudioControlObject::GetNumSources (UInt8 unitID) {
+    USBACDescriptorObject *				thisProcessingUnit;
+    UInt8								numSources;
+
+    numSources = 0;
+    thisProcessingUnit = GetACDescriptorObject (unitID);
+    if (thisProcessingUnit)
+		numSources = thisProcessingUnit->GetNumInPins ();
+
+    return numSources;
+}
+
+UInt8 USBAudioControlObject::GetSourceID (UInt8 unitID) {
+	USBACDescriptorObject *				theUnit;
+	UInt8								sourceID;
+
+	sourceID = 0;
+	theUnit = GetACDescriptorObject (unitID);
+	if (NULL != theUnit) {
+		sourceID = theUnit->GetSourceID ();
+	}
+
+	return sourceID;
+}
+
+UInt8 * USBAudioControlObject::GetSourceIDs (UInt8 unitID) {
+	USBACDescriptorObject *				theUnit;
+	UInt8 *								sourceIDs;
+
+	sourceIDs = NULL;
+	theUnit = GetACDescriptorObject (unitID);
+	if (NULL != theUnit) {
+		switch (theUnit->GetDescriptorSubType ()) {
+			case MIXER_UNIT:
+				sourceIDs = GetMixerSources (unitID);
+				break;
+			case SELECTOR_UNIT:
+				sourceIDs = GetSelectorSources (unitID);
+				break;
+			case PROCESSING_UNIT:
+				sourceIDs = GetProcessingUnitSources (unitID);
+				break;
+			case EXTENSION_UNIT:
+				sourceIDs = GetExtensionUnitSources (unitID);
+				break;
+		}
+	}
+
+	return sourceIDs;
+}
+
+UInt8 USBAudioControlObject::GetSubType (UInt8 unitID) {
+	USBACDescriptorObject *				theUnit;
+	UInt8								subType;
+
+	subType = 0;
+	theUnit = GetACDescriptorObject (unitID);
+	if (NULL != theUnit) {
+		subType = theUnit->GetDescriptorSubType ();
+	}
+
+	return subType;
+}
+
+USBProcessingUnitObject * USBAudioControlObject::GetProcessingUnitObject (UInt8 unitID) {
+    USBProcessingUnitObject * 			thisProcessingUnit;
+    UInt8								indx;
+    Boolean								found;
+
+    indx = 0;
+    found = FALSE;
+	thisProcessingUnit = NULL;
+	if (NULL != theProcessingUnits) {
+		while (!found && indx < theProcessingUnits->getCount ()) {
+			thisProcessingUnit = OSDynamicCast (USBProcessingUnitObject, theProcessingUnits->getObject (indx));
+			if (thisProcessingUnit && unitID == thisProcessingUnit->GetUnitID ()) {
+				found = TRUE;
+			}
+			indx++;
+		}
+	}
+
+	if (found != TRUE)
+		thisProcessingUnit = NULL;
+
+    return thisProcessingUnit;
 }
 
 USBMixerUnitObject * USBAudioControlObject::GetMixerObject (UInt8 unitID) {
@@ -1503,18 +1717,18 @@ USBMixerUnitObject * USBAudioControlObject::GetMixerObject (UInt8 unitID) {
     return thisMixerObject;
 }
 
-USBOutputTerminalObject * USBAudioControlObject::GetOutputTerminalObject (UInt8 unitID) {
-    USBOutputTerminalObject * 			thisOutputTerminal;
+USBExtensionUnitObject * USBAudioControlObject::GetExtensionUnitObject (UInt8 unitID) {
+    USBExtensionUnitObject * 			thisExtensionUnit;
     UInt8								indx;
     Boolean								found;
 
     indx = 0;
     found = FALSE;
-	thisOutputTerminal = NULL;
-	if (NULL != theOutputTerminals) {
-		while (!found && indx < theOutputTerminals->getCount ()) {
-			thisOutputTerminal = OSDynamicCast (USBOutputTerminalObject, theOutputTerminals->getObject (indx));
-			if (thisOutputTerminal && unitID == thisOutputTerminal->GetUnitID ()) {
+	thisExtensionUnit = NULL;
+	if (NULL != theExtensionUnits) {
+		while (!found && indx < theExtensionUnits->getCount ()) {
+			thisExtensionUnit = OSDynamicCast (USBExtensionUnitObject, theExtensionUnits->getObject (indx));
+			if (thisExtensionUnit && unitID == thisExtensionUnit->GetUnitID ()) {
 				found = TRUE;
 			}
 			indx++;
@@ -1522,9 +1736,9 @@ USBOutputTerminalObject * USBAudioControlObject::GetOutputTerminalObject (UInt8
 	}
 
 	if (found != TRUE)
-		thisOutputTerminal = NULL;
+		thisExtensionUnit = NULL;
 
-    return thisOutputTerminal;
+    return thisExtensionUnit;
 }
 
 USBSelectorUnitObject * USBAudioControlObject::GetSelectorUnitObject (UInt8 unitID) {
@@ -1575,6 +1789,42 @@ UInt8 * USBAudioControlObject::GetSelectorSources (UInt8 unitID) {
     return selectorSources;
 }
 
+UInt8 * USBAudioControlObject::GetMixerSources (UInt8 unitID) {
+    USBMixerUnitObject *				thisMixerUnit;
+    UInt8 *								mixerSources;
+
+    mixerSources = NULL;
+    thisMixerUnit = GetMixerObject (unitID);
+    if (thisMixerUnit)
+		mixerSources = thisMixerUnit->GetSources ();
+
+    return mixerSources;
+}
+
+UInt8 * USBAudioControlObject::GetExtensionUnitSources (UInt8 unitID) {
+    USBExtensionUnitObject *			thisExtensionUnit;
+    UInt8 *								extensionUnitSources;
+
+    extensionUnitSources = NULL;
+    thisExtensionUnit = GetExtensionUnitObject (unitID);
+    if (thisExtensionUnit)
+		extensionUnitSources = thisExtensionUnit->GetSources ();
+
+    return extensionUnitSources;
+}
+
+UInt8 * USBAudioControlObject::GetProcessingUnitSources (UInt8 unitID) {
+    USBProcessingUnitObject *			thisProcessingUnit;
+    UInt8 *								processingUnitSources;
+
+    processingUnitSources = NULL;
+    thisProcessingUnit = GetProcessingUnitObject (unitID);
+    if (thisProcessingUnit)
+		processingUnitSources = thisProcessingUnit->GetSources ();
+
+    return processingUnitSources;
+}
+
 Boolean USBAudioControlObject::MasterHasMuteControl (UInt8 featureUnitID) {
 	USBFeatureUnitObject *				thisFeatureUnit;
 	Boolean								result;
@@ -1593,6 +1843,8 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 	USBFeatureUnitObject *		featureUnit;
 	USBMixerUnitObject *		mixerUnit;
 	USBSelectorUnitObject *		selectorUnit;
+	USBProcessingUnitObject *	processingUnit;
+	USBExtensionUnitObject *	extensionUnit;
 	UInt8						index;
 
 	debugIOLog ("+USBAudioControlObject::ParseACInterfaceDescriptor (%p, %d)", theInterfacePtr, currentInterface);
@@ -1623,6 +1875,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
                 debugIOLog ("in INPUT_TERMINAL in ParseACInterfaceDescriptor");
 				inputTerminal = new USBInputTerminalObject;
 				FailIf (NULL == inputTerminal, Exit);
+				inputTerminal->SetDescriptorSubType (theInterfacePtr->bDescriptorSubtype);
                 inputTerminal->SetUnitID (((ACInputTerminalDescriptorPtr)theInterfacePtr)->bTerminalID);
                 inputTerminal->SetTerminalType (USBToHostWord (((ACInputTerminalDescriptorPtr)theInterfacePtr)->wTerminalType));
                 inputTerminal->SetAssocTerminal (((ACInputTerminalDescriptorPtr)theInterfacePtr)->bAssocTerminal);
@@ -1642,6 +1895,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
                 debugIOLog ("in OUTPUT_TERMINAL in ParseACInterfaceDescriptor");
 				outputTerminal = new USBOutputTerminalObject;
 				FailIf (NULL == outputTerminal, Exit);
+				outputTerminal->SetDescriptorSubType (theInterfacePtr->bDescriptorSubtype);
                 outputTerminal->SetUnitID (((ACOutputTerminalDescriptorPtr)theInterfacePtr)->bTerminalID);
                 outputTerminal->SetTerminalType (USBToHostWord (((ACOutputTerminalDescriptorPtr)theInterfacePtr)->wTerminalType));
                 outputTerminal->SetAssocTerminal (((ACOutputTerminalDescriptorPtr)theInterfacePtr)->bAssocTerminal);
@@ -1663,6 +1917,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
                 debugIOLog ("in FEATURE_UNIT in ParseACInterfaceDescriptor");
 				featureUnit = new USBFeatureUnitObject;
 				FailIf (NULL == featureUnit, Exit);
+				featureUnit->SetDescriptorSubType (theInterfacePtr->bDescriptorSubtype);
                 featureUnit->SetUnitID (((ACFeatureUnitDescriptorPtr)theInterfacePtr)->bUnitID);
                 featureUnit->SetSourceID (((ACFeatureUnitDescriptorPtr)theInterfacePtr)->bSourceID);
                 featureUnit->SetControlSize (((ACFeatureUnitDescriptorPtr)theInterfacePtr)->bControlSize);
@@ -1691,6 +1946,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				mixerUnit = new USBMixerUnitObject;
 				FailIf (NULL == mixerUnit, Exit);
 				debugIOLog ("descriptor length = %d", theInterfacePtr->bLength);
+				mixerUnit->SetDescriptorSubType (theInterfacePtr->bDescriptorSubtype);
 				mixerUnit->SetUnitID (((ACMixerUnitDescriptorPtr)theInterfacePtr)->bUnitID);
 				debugIOLog ("unit ID = %d", ((ACMixerUnitDescriptorPtr)theInterfacePtr)->bUnitID);
 				debugIOLog ("numInPins = %d", ((ACMixerUnitDescriptorPtr)theInterfacePtr)->bNrInPins);
@@ -1720,6 +1976,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
                 debugIOLog ("in SELECTOR_UNIT in ParseACInterfaceDescriptor");
 				selectorUnit = new USBSelectorUnitObject;
 				FailIf (NULL == selectorUnit, Exit);
+				selectorUnit->SetDescriptorSubType (theInterfacePtr->bDescriptorSubtype);
 				selectorUnit->SetUnitID (((ACSelectorUnitDescriptorPtr)theInterfacePtr)->bUnitID);
 				selectorUnit->InitSourceIDs (&((ACSelectorUnitDescriptorPtr)theInterfacePtr)->baSourceID[0], ((ACSelectorUnitDescriptorPtr)theInterfacePtr)->bNrInPins);
 				debugIOLog ("numInPins on selector = %d", ((ACSelectorUnitDescriptorPtr)theInterfacePtr)->bNrInPins);
@@ -1734,10 +1991,57 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				selectorUnit->release ();
 				break;
             case PROCESSING_UNIT:
+				{
+				UInt16 *					channelConfig;
+				UInt8						controlSize;
+				UInt8						nrChannels;
+
                 debugIOLog ("in PROCESSING_UNIT in ParseACInterfaceDescriptor");
+				processingUnit = new USBProcessingUnitObject;
+				FailIf (NULL == processingUnit, Exit);
+				processingUnit->SetDescriptorSubType (theInterfacePtr->bDescriptorSubtype);
+				processingUnit->SetUnitID (((ACProcessingUnitDescriptorPtr)theInterfacePtr)->bUnitID);
+				processingUnit->SetProcessType (((ACProcessingUnitDescriptorPtr)theInterfacePtr)->wProcessType);
+				debugIOLog ("processing unit type = 0x%x", ((ACProcessingUnitDescriptorPtr)theInterfacePtr)->wProcessType);
+				processingUnit->InitSourceIDs (&((ACProcessingUnitDescriptorPtr)theInterfacePtr)->baSourceID[0], ((ACProcessingUnitDescriptorPtr)theInterfacePtr)->bNrInPins);
+				debugIOLog ("numInPins = %d", ((ACProcessingUnitDescriptorPtr)theInterfacePtr)->bNrInPins);
+				nrChannels = ((ACProcessingUnitDescriptorPtr)theInterfacePtr)->baSourceID[((ACProcessingUnitDescriptorPtr)theInterfacePtr)->bNrInPins];
+				debugIOLog ("nrChannels = %d", nrChannels);
+				processingUnit->SetNumChannels (nrChannels);
+				channelConfig = (UInt16 *)&((ACProcessingUnitDescriptorPtr)theInterfacePtr)->baSourceID[((ACProcessingUnitDescriptorPtr)theInterfacePtr)->bNrInPins + 1];
+				*channelConfig = USBToHostWord (*channelConfig);
+				debugIOLog ("channelConfig = %d", *channelConfig);
+				processingUnit->SetChannelConfig (*channelConfig);
+				controlSize = ((ACProcessingUnitDescriptorPtr)theInterfacePtr)->baSourceID[((ACProcessingUnitDescriptorPtr)theInterfacePtr)->bNrInPins + 4];
+				debugIOLog ("controlSize = %d", controlSize);
+				processingUnit->InitControlsArray (&((ACProcessingUnitDescriptorPtr)theInterfacePtr)->baSourceID[((ACProcessingUnitDescriptorPtr)theInterfacePtr)->bNrInPins + 5], controlSize);
+
+				if (NULL == theProcessingUnits) {
+					theProcessingUnits = OSArray::withObjects ((const OSObject **)&processingUnit, 1);
+					FailIf (NULL == theProcessingUnits, Exit);
+				} else {
+					theProcessingUnits->setObject (processingUnit);
+				}
+
+				processingUnit->release ();
 				break;
+				}
             case EXTENSION_UNIT:
                 debugIOLog ("in EXTENSION_UNIT in ParseACInterfaceDescriptor");
+				extensionUnit = new USBExtensionUnitObject;
+				FailIf (NULL == extensionUnit, Exit);
+				extensionUnit->SetDescriptorSubType (theInterfacePtr->bDescriptorSubtype);
+				extensionUnit->SetUnitID (((ACExtensionUnitDescriptorPtr)theInterfacePtr)->bUnitID);
+				extensionUnit->InitSourceIDs (&((ACExtensionUnitDescriptorPtr)theInterfacePtr)->baSourceID[0], ((ACExtensionUnitDescriptorPtr)theInterfacePtr)->bNrInPins);
+
+				if (NULL == theExtensionUnits) {
+					theExtensionUnits = OSArray::withObjects ((const OSObject **)&extensionUnit, 1);
+					FailIf (NULL == theExtensionUnits, Exit);
+				} else {
+					theExtensionUnits->setObject (extensionUnit);
+				}
+
+				extensionUnit->release ();
 				break;
             default:
                 debugIOLog ("in default in ParseACInterfaceDescriptor");
@@ -2164,16 +2468,20 @@ void USBMixerUnitObject::free (void) {
 }
 
 void USBMixerUnitObject::InitControlsArray (UInt8 * bmCntrls, UInt8 bmControlSize) {
+#if DEBUGLOGGING
 	UInt8					i;
+#endif
 
 	controlSize = bmControlSize;
 	bmControls = ((UInt8 *)IOMalloc (bmControlSize));
 	FailIf (NULL == bmControls, Exit);
 	memcpy (bmControls, bmCntrls, bmControlSize);
+#if DEBUGLOGGING
 	for (i = 0; i < bmControlSize; i++) {
 		debugIOLog ("bmCntrls[%d] = 0x%X", i, bmCntrls[i]);
 		debugIOLog ("bmControls[%d] = 0x%X", i, bmControls[i]);
 	}
+#endif
 Exit:
 	return;
 }
@@ -2184,7 +2492,7 @@ void USBMixerUnitObject::InitSourceIDs (UInt8 * baSrcIDs, UInt8 nrInPins) {
 	FailIf (NULL == baSourceIDs, Exit);
 	memcpy (baSourceIDs, baSrcIDs, nrInPins);
 
-#if DEBUGLOG
+#if DEBUGLOGGING
 	{
 		UInt8			i;
 		for (i = 0; i < nrInPins; i++) {
@@ -2214,6 +2522,38 @@ Exit:
 	return;
 }
 
+/* ------------------------------------------------------
+    USBProcessingUnitObject
+------------------------------------------------------ */
+void USBProcessingUnitObject::free (void) {
+    USBACDescriptorObject::free ();
+}
+
+void USBProcessingUnitObject::InitSourceIDs (UInt8 * baSrcIDs, UInt8 nrInPins) {
+	numInPins = nrInPins;
+	baSourceIDs = (UInt8 *)IOMalloc (nrInPins);
+	FailIf (NULL == baSourceIDs, Exit);
+	memcpy (baSourceIDs, baSrcIDs, nrInPins);
+
+Exit:
+	return;
+}
+
+void USBProcessingUnitObject::InitControlsArray (UInt8 * bmCntrls, UInt8 bmControlSize) {
+	UInt8					i;
+
+	controlSize = bmControlSize;
+	bmControls = ((UInt8 *)IOMalloc (bmControlSize));
+	FailIf (NULL == bmControls, Exit);
+	memcpy (bmControls, bmCntrls, bmControlSize);
+	for (i = 0; i < bmControlSize; i++) {
+		debugIOLog ("bmCntrls[%d] = 0x%X", i, bmCntrls[i]);
+		debugIOLog ("bmControls[%d] = 0x%X", i, bmControls[i]);
+	}
+Exit:
+	return;
+}
+
 /* ------------------------------------------------------
     USBFeatureUnitObject
 ------------------------------------------------------ */
@@ -2276,6 +2616,47 @@ Exit:
 	return;
 }
 
+/* ------------------------------------------------------
+    USBExtensionUnitObject
+------------------------------------------------------ */
+void USBExtensionUnitObject::free (void) {
+    USBACDescriptorObject::free ();
+}
+
+void USBExtensionUnitObject::InitControlsArray (UInt8 * bmCntrls, UInt8 bmControlSize) {
+	UInt8					i;
+
+	controlSize = bmControlSize;
+	bmControls = ((UInt8 *)IOMalloc (bmControlSize));
+	FailIf (NULL == bmControls, Exit);
+	memcpy (bmControls, bmCntrls, bmControlSize);
+	for (i = 0; i < bmControlSize; i++) {
+		debugIOLog ("bmCntrls[%d] = 0x%X", i, bmCntrls[i]);
+		debugIOLog ("bmControls[%d] = 0x%X", i, bmControls[i]);
+	}
+Exit:
+	return;
+}
+
+void USBExtensionUnitObject::InitSourceIDs (UInt8 * baSrcIDs, UInt8 nrInPins) {
+	numInPins = nrInPins;
+	baSourceIDs = (UInt8 *)IOMalloc (nrInPins);
+	FailIf (NULL == baSourceIDs, Exit);
+	memcpy (baSourceIDs, baSrcIDs, nrInPins);
+
+#if DEBUGLOGGING
+	{
+		UInt8			i;
+		for (i = 0; i < nrInPins; i++) {
+			debugIOLog ("baSourceIDs[%d] = %d", i, baSourceIDs[i]);
+		}
+	}
+#endif
+
+Exit:
+	return;
+}
+
 /* ------------------------------------------------------
     USBACDescriptorObject
 ------------------------------------------------------ */
